#define LAYER2_C

#include <string.h>
#include <float.h>
#include "layer2.h"
#include "memory_stream.h"

void init_layer2_work(LAYER2_WORK *work);
int parse_layer2_header(unsigned int sync, LAYER2_HEADER *out);
int decode_layer2(LAYER2_HEADER *head, unsigned char *buffer, LAYER2_WORK *work, short *pcm);

static int decode_layer2_stereo(LAYER2_HEADER *head, unsigned char *buffer, LAYER2_WORK *work, short *pcm);
static int decode_layer2_monaural(LAYER2_HEADER *head, unsigned char *buffer, LAYER2_WORK *work, short *pcm);

void init_layer2_work(LAYER2_WORK *work)
{
	memset(work->area, 0, sizeof(work->area));
	work->offset = 0;
}

int parse_layer2_header(unsigned int sync, LAYER2_HEADER *out)
{
	int channel_bitrate;
	static const int sblimit = 32;
	
	static const int bitrate[16] = {
		-1, 32000, 48000, 56000, 64000, 80000, 96000, 112000,
		128000, 160000, 192000, 224000, 256000, 320000, 384000, 0
	};

	static const int frequency[4] = {
		44100, 48000, 32000, 0,
	};

	if( (sync & 0xfff00000) != 0xfff00000 ){
		goto LAYER2_PARSE_HEADER_ERROR;
	}

	if( ((sync >> 19) & 1) == 1 ){
		out->version = 1;
	}

	switch((sync >> 17) & 3){
	case 0:
		/* undefined layer */
		goto LAYER2_PARSE_HEADER_ERROR;
	case 1:
		/* layer3 */
		goto LAYER2_PARSE_HEADER_ERROR;
	case 2:
		out->layer = 2;
		break;
	case 3:
		/* layer1 */
		goto LAYER2_PARSE_HEADER_ERROR;
	}

	out->has_crc = ((sync >> 16) & 1) ^ 1;

	out->bitrate = bitrate[(sync >> 12) & 0xf];
	if(out->bitrate < 1){
		goto LAYER2_PARSE_HEADER_ERROR;
	}
	out->frequency = frequency[(sync >> 10) & 3];
	if(out->frequency == 0){
		goto LAYER2_PARSE_HEADER_ERROR;
	}
	out->padding = (sync >> 9) & 1;

	switch((sync >> 6) & 3){
	case 0:
		/* stereo */
		out->channel = 2;
		out->bound = sblimit;
		break;
	case 1:
		/* intensity stereo */
		out->channel = 2;
		out->bound = (((sync >> 4) & 3) + 1) << 2;
		break;
	case 2:
		/* dual channel */
		out->channel = 2;
		out->bound = sblimit;
		break;
	case 3:
		/* monaural */
		out->channel = 1;
		out->bound = sblimit;
		break;
	}

	out->emphasis = sync & 3;

	if(out->bitrate < 1){
		/* free format is not supported */
		goto LAYER2_PARSE_HEADER_ERROR;
	}

	if(out->channel == 2){
		channel_bitrate = out->bitrate / 2;
		if(channel_bitrate < 32000){
			/* unsupported bitrate */
			goto LAYER2_PARSE_HEADER_ERROR;
		}
	}else{
		channel_bitrate = out->bitrate;
		if(channel_bitrate > 192000){
			/* unsupported bitrate */
			goto LAYER2_PARSE_HEADER_ERROR;
		}
	}

	if(channel_bitrate < 56000){
		if(out->frequency == 32000){
			out->sblimit = 12;
		}else{
			out->sblimit = 8;
		}
	}else if(channel_bitrate < 96000){
		out->sblimit = 27;
	}else{
		if(out->frequency == 48000){
			out->sblimit = 27;
		}else{
			out->sblimit = 30;
		}
	}
	if(out->bound > out->sblimit){
		out->bound = out->sblimit;
	}
	
	switch(out->version){
	case 1:
		out->framesize = 144*out->bitrate/out->frequency+out->padding;
		break;
	default: 
		/* only support MPEG-1 */
		goto LAYER2_PARSE_HEADER_ERROR; 
	}

	return 1;
	
LAYER2_PARSE_HEADER_ERROR:
	memset(out, 0, sizeof(LAYER2_HEADER));
	return 0;
}

static const double table_b1[] = {
	2.00000000000000, 1.58740105196820, 1.25992104989487, 1.00000000000000,
	0.79370042498410, 0.62996052494744, 0.50000000000000, 0.39685026299205,
	0.31498026247372, 0.25000000000000, 0.19842513149602, 0.15749013123686,
	0.12500000000000, 0.09921256574801, 0.07874506561843, 0.06250000000000,

	0.04960628287401, 0.03937253280921, 0.03125000000000, 0.02480314143700,
	0.01968626640461, 0.01562500000000, 0.01240157071850, 0.00984313320230,
	0.00781250000000, 0.00620078535925, 0.00492156660115, 0.00390625000000,
	0.00310039267963, 0.00246078330058, 0.00195312500000, 0.00155019633981,
	
	0.00123039165029, 0.00097656250000, 0.00077509816991, 0.00061519582514,
	0.00048828125000, 0.00038754908495, 0.00030759791257, 0.00024414062500,
	0.00019377454248, 0.00015379895629, 0.00012207031250, 0.00009688727124,
	0.00007689947814, 0.00006103515625, 0.00004844363562, 0.00003844973907,

	0.00003051757813, 0.00002422181781, 0.00001922486954, 0.00001525878906,
	0.00001211090890, 0.00000961243477, 0.00000762939453, 0.00000605545445,
	0.00000480621738, 0.00000381469727, 0.00000302772723, 0.00000240310869,
	0.00000190734863, 0.00000151386361, 0.00000120155435, 0.0
};
	
static const int table_b2ab_nbal[] = {
	4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 2,
	2, 2, 2, 2, 2, 2, 0, 0,
};

static const int table_b2cd_nbal[] = {
	4, 4, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 0, 0, 0, 0,
};

static const int table_b2_4a[] = {
	0,  1,  4,  5,  6,  7,  8,  9,
	10, 11, 12, 13, 14, 15, 16, 17,
};

static const int table_b2_4b[] = {
	0,  1,  2,  4,  3,  5,  6,  7,
	8,  9, 10, 11, 12, 13, 14, 17,
};

static const int table_b2_4c[] = {
	0,  1,  2,  3,  5,  6,  7,  8,
	9, 10, 11, 12, 13, 14, 15, 16,
};

static const int table_b2_3a[] = {
	0, 1, 2, 4, 3, 5, 6, 17,
};

static const int table_b2_3b[] = {
	0, 1, 2, 3, 5, 6, 7, 8,
};

static const int table_b2_2[] = {
	0, 1, 2, 17,
};

static const int *table_b2ab[] = {
	table_b2_4a, table_b2_4a, table_b2_4a, table_b2_4b,
	table_b2_4b, table_b2_4b, table_b2_4b, table_b2_4b,

	table_b2_4b, table_b2_4b, table_b2_4b, table_b2_3a,
	table_b2_3a, table_b2_3a, table_b2_3a, table_b2_3a,

	table_b2_3a, table_b2_3a, table_b2_3a, table_b2_3a,
	table_b2_3a, table_b2_3a, table_b2_3a, table_b2_2,

	table_b2_2,  table_b2_2,  table_b2_2,  table_b2_2,
	table_b2_2,  table_b2_2,  table_b2_2,  table_b2_2,
};

static const int *table_b2cd[] = {
	table_b2_4c, table_b2_4c, table_b2_3b, table_b2_3b,
	table_b2_3b, table_b2_3b, table_b2_3b, table_b2_3b,
	table_b2_3b, table_b2_3b, table_b2_3b, table_b2_3b,
};

static const double table_b3[] = {
	 0.000000000, -0.000015259, -0.000015259, -0.000015259,
	-0.000015259, -0.000015259, -0.000015259, -0.000030518,
	-0.000030518, -0.000030518, -0.000030518, -0.000045776,
	-0.000045776, -0.000061035, -0.000061035, -0.000076294, /* 15 */

	-0.000076294, -0.000091553, -0.000106812, -0.000106812,
	-0.000122070, -0.000137329, -0.000152588, -0.000167847,
	-0.000198364, -0.000213623, -0.000244141, -0.000259399,
	-0.000289917, -0.000320435, -0.000366211, -0.000396729, /* 31 */

	-0.000442505, -0.000473022, -0.000534058, -0.000579834,
	-0.000625610, -0.000686646, -0.000747681, -0.000808716,
	-0.000885010, -0.000961304, -0.001037598, -0.001113892,
	-0.001205444, -0.001296997, -0.001388550, -0.001480103, /* 47 */

	-0.001586914, -0.001693726, -0.001785278, -0.001907349,
	-0.002014160, -0.002120972, -0.002243042, -0.002349854,
	-0.002456665, -0.002578735, -0.002685547, -0.002792358,
	-0.002899170, -0.002990723, -0.003082275, -0.003173828, /* 63 */

	 0.003250122,  0.003326416,  0.003387451,  0.003433228,
	 0.003463745,  0.003479004,  0.003479004,  0.003463745,
	 0.003417969,  0.003372192,  0.003280640,  0.003173828,
	 0.003051758,  0.002883911,  0.002700806,  0.002487183, /* 79 */

	 0.002227783,  0.001937866,  0.001617432,  0.001266479,
	 0.000869751,  0.000442505, -0.000030518, -0.000549316,
	-0.001098633, -0.001693726, -0.002334595, -0.003005981,
	-0.003723145, -0.004486084, -0.005294800, -0.006118774, /* 95 */

	-0.007003784, -0.007919312, -0.008865356, -0.009841919,
	-0.010848999, -0.011886597, -0.012939453, -0.014022827,
	-0.015121460, -0.016235352, -0.017349243, -0.018463135,
	-0.019577026, -0.020690918, -0.021789551, -0.022857666, /* 111 */

	-0.023910522, -0.024932861, -0.025909424, -0.026840210,
	-0.027725220, -0.028533936, -0.029281616, -0.029937744,
	-0.030532837, -0.031005859, -0.031387329, -0.031661987,
	-0.031814575, -0.031845093, -0.031738281, -0.031478882, /* 127 */

	 0.031082153,  0.030517578,  0.029785156,  0.028884888,
	 0.027801514,  0.026535034,  0.025085449,  0.023422241,
	 0.021575928,  0.019531250,  0.017257690,  0.014801025,
	 0.012115479,  0.009231567,  0.006134033,  0.002822876, /* 143 */

	-0.000686646, -0.004394531, -0.008316040, -0.012420654,
	-0.016708374, -0.021179199, -0.025817871, -0.030609131,
	-0.035552979, -0.040634155, -0.045837402, -0.051132202,
	-0.056533813, -0.061996460, -0.067520142, -0.073059082, /* 159 */

	-0.078628540, -0.084182739, -0.089706421, -0.095169067,
	-0.100540161, -0.105819702, -0.110946655, -0.115921021,
	-0.120697021, -0.125259399, -0.129562378, -0.133590698,
	-0.137298584, -0.140670776, -0.143676758, -0.146255493, /* 175 */

	-0.148422241, -0.150115967, -0.151306152, -0.151962280,
	-0.152069092, -0.151596069, -0.150497437, -0.148773193,
	-0.146362305, -0.143264771, -0.139450073, -0.134887695,
	-0.129577637, -0.123474121, -0.116577148, -0.108856201, /* 191 */

	 0.100311279,  0.090927124,  0.080688477,  0.069595337,
	 0.057617187,  0.044784546,  0.031082153,  0.016510010,
	 0.001068115, -0.015228271, -0.032379150, -0.050354004,
	-0.069168091, -0.088775635, -0.109161377, -0.130310059, /* 207 */

	-0.152206421, -0.174789429, -0.198059082, -0.221984863,
	-0.246505737, -0.271591187, -0.297210693, -0.323318481,
	-0.349868774, -0.376800537, -0.404083252, -0.431655884,
	-0.459472656, -0.487472534, -0.515609741, -0.543823242, /* 223 */

	-0.572036743, -0.600219727, -0.628295898, -0.656219482,
	-0.683914185, -0.711318970, -0.738372803, -0.765029907,
	-0.791213989, -0.816864014, -0.841949463, -0.866363525,
	-0.890090942, -0.913055420, -0.935195923, -0.956481934, /* 239 */

	-0.976852417, -0.996246338, -1.014617920, -1.031936646,
	-1.048156738, -1.063217163, -1.077117920, -1.089782715,
	-1.101211548, -1.111373901, -1.120223999, -1.127746582,
	-1.133926392, -1.138763428, -1.142211914, -1.144287109, /* 255 */

	 1.144989014,  1.144287109,  1.142211914,  1.138763428,
	 1.133926392,  1.127746582,  1.120223999,  1.111373901,
	 1.101211548,  1.089782715,  1.077117920,  1.063217163,
	 1.048156738,  1.031936646,  1.014617920,  0.996246338, /* 271 */

	 0.976852417,  0.956481934,  0.935195923,  0.913055420,
	 0.890090942,  0.866363525,  0.841949463,  0.816864014,
	 0.791213989,  0.765029907,  0.738372803,  0.711318970,
	 0.683914185,  0.656219482,  0.628295898,  0.600219727, /* 287 */

	 0.572036743,  0.543823242,  0.515609741,  0.487472534,
	 0.459472656,  0.431655884,  0.404083252,  0.376800537,
	 0.349868774,  0.323318481,  0.297210693,  0.271591187,
	 0.246505737,  0.221984863,  0.198059082,  0.174789429, /* 304 */

	 0.152206421,  0.130310059,  0.109161377,  0.088775635,
	 0.069168091,  0.050354004,  0.032379150,  0.015228271,
	-0.001068115, -0.016510010, -0.031082153, -0.044784546,
	-0.057617187, -0.069595337, -0.080688477, -0.090927124, /* 319 */

	 0.100311279,  0.108856201,  0.116577148,  0.123474121,
	 0.129577637,  0.134887695,  0.139450073,  0.143264771,
	 0.146362305,  0.148773193,  0.150497437,  0.151596069,
	 0.152069092,  0.151962280,  0.151306152,  0.150115967, /* 335 */

	 0.148422241,  0.146255493,  0.143676758,  0.140670776,
	 0.137298584,  0.133590698,  0.129562378,  0.125259399,
	 0.120697021,  0.115921021,  0.110946655,  0.105819702,
	 0.100540161,  0.095169067,  0.089706421,  0.084182739, /* 351 */

	 0.078628540,  0.073059082,  0.067520142,  0.061996460,
	 0.056533813,  0.051132202,  0.045837402,  0.040634155,
	 0.035552979,  0.030609131,  0.025817871,  0.021179199,
	 0.016708374,  0.012420654,  0.008316040,  0.004394531, /* 367 */

	 0.000686646, -0.002822876, -0.006134033, -0.009231567,
	-0.012115479, -0.014801025, -0.017257690, -0.019531250,
	-0.021575928, -0.023422241, -0.025085449, -0.026535034,
	-0.027801514, -0.028884888, -0.029785156, -0.030517578, /* 383 */

	 0.031082153,  0.031478882,  0.031738281,  0.031845093,
	 0.031814575,  0.031661987,  0.031387329,  0.031005859,
	 0.030532837,  0.029937744,  0.029281616,  0.028533936,
	 0.027725220,  0.026840210,  0.025909424,  0.024932861, /* 399 */

	 0.023910522,  0.022857666,  0.021789551,  0.020690918,
	 0.019577026,  0.018463135,  0.017349243,  0.016235352,
	 0.015121460,  0.014022827,  0.012939453,  0.011886597,
	 0.010848999,  0.009841919,  0.008865356,  0.007919312, /* 415 */

	 0.007003784,  0.006118774,  0.005294800,  0.004486084,
	 0.003723145,  0.003005981,  0.002334595,  0.001693726,
	 0.001098633,  0.000549316,  0.000030518, -0.000442505,
	-0.000869751, -0.001266479, -0.001617432, -0.001937866, /* 431 */

	-0.002227783, -0.002487183, -0.002700806, -0.002883911,
	-0.003051758, -0.003173828, -0.003280640, -0.003372192,
	-0.003417969, -0.003463745, -0.003479004, -0.003479004,
	-0.003463745, -0.003433228, -0.003387451, -0.003326416, /* 447 */

	 0.003250122,  0.003173828,  0.003082275,  0.002990723,
	 0.002899170,  0.002792358,  0.002685547,  0.002578735,
	 0.002456665,  0.002349854,  0.002243042,  0.002120972,
	 0.002014160,  0.001907349,  0.001785278,  0.001693726, /* 463 */

	 0.001586914,  0.001480103,  0.001388550,  0.001296997,
	 0.001205444,  0.001113892,  0.001037598,  0.000961304,
	 0.000885010,  0.000808716,  0.000747681,  0.000686646,
	 0.000625610,  0.000579834,  0.000534058,  0.000473022, /* 479 */

	 0.000442505,  0.000396729,  0.000366211,  0.000320435,
	 0.000289917,  0.000259399,  0.000244141,  0.000213623,
	 0.000198364,  0.000167847,  0.000152588,  0.000137329,
	 0.000122070,  0.000106812,  0.000106812,  0.000091553, /* 495 */

	 0.000076294,  0.000076294,  0.000061035,  0.000061035,
	 0.000045776,  0.000045776,  0.000030518,  0.000030518,
	 0.000030518,  0.000030518,  0.000015259,  0.000015259,
	 0.000015259,  0.000015259,  0.000015259,  0.000015259, /* 511 */
};

typedef struct {
	double       c;
	double       d;
	int          bits;
	int          step; /* denominator */
} TABLE_B4_ELEMENT;

static const TABLE_B4_ELEMENT table_b4[] = {
	{0.0,             0.0,          0,     0},
	{1.0+1.0/3.0,     1.0/2.0,      5,     3},
	{1.0+3.0/5.0,     1.0/2.0,      7,     5},
	{1.0+7.0/9.0,     1.0/2.0,     10,     9},
	{1.0+1.0/7.0,     1.0/4.0,      3,     4},
	{1.0+1.0/15.0,    1.0/8.0,      4,     8},
	{1.0+1.0/31.0,    1.0/16.0,     5,    16},
	{1.0+1.0/63.0,    1.0/32.0,     6,    32},
	{1.0+1.0/127.0,   1.0/64.0,     7,    64},
	{1.0+1.0/255.0,   1.0/128.0,    8,   128},
	{1.0+1.0/512.0,   1.0/256.0,    9,   256},
	{1.0+1.0/1023.0,  1.0/512.0,   10,   512},
	{1.0+1.0/2047.0,  1.0/1024.0,  11,  1024},
	{1.0+1.0/4095.0,  1.0/2048.0,  12,  2048},
	{1.0+1.0/9181.0,  1.0/4096.0,  13,  4096},
	{1.0+1.0/16383.0, 1.0/8192.0,  14,  8192},
	{1.0+1.0/32767.0, 1.0/16384.0, 15, 16384},
	{1.0+1.0/65535.0, 1.0/32768.0, 16, 32768},
};

static const double table_Nik[64][32] = {
	{
		 0.707106781186547573, -0.707106781186547462, -0.707106781186547684,  0.707106781186547351, 
		 0.707106781186547684, -0.707106781186547906, -0.707106781186547129,  0.707106781186547795, 
		 0.707106781186547240, -0.707106781186547795, -0.707106781186547351,  0.707106781186546352, 
		 0.707106781186547462, -0.707106781186546352, -0.707106781186547573,  0.707106781186548683, 
		 0.707106781186547573, -0.707106781186548683, -0.707106781186547684,  0.707106781186548572, 
		 0.707106781186545241, -0.707106781186546018, -0.707106781186547906,  0.707106781186548350, 
		 0.707106781186545463, -0.707106781186545796, -0.707106781186548017,  0.707106781186548239, 
		 0.707106781186545574, -0.707106781186545574, -0.707106781186548239,  0.707106781186548017, 
	},{
		 0.671558954847018330, -0.803207531480644832, -0.514102744193221772,  0.903989293123443449, 
		 0.336889853392220051, -0.970031253194544085, -0.146730474455361665,  0.998795456205172405, 
		-0.049067674327417293, -0.989176509964780903,  0.242980179903262428,  0.941544065183020806, 
		-0.427555093430283195, -0.857728610000272562,  0.595699304492433690,  0.740951125354960216, 
		-0.740951125354958884, -0.595699304492432469,  0.857728610000271452,  0.427555093430285082, 
		-0.941544065183021361, -0.242980179903264426,  0.989176509964780570,  0.049067674327415829, 
		-0.998795456205172405,  0.146730474455359611,  0.970031253194543308, -0.336889853392220606, 
		-0.903989293123444004,  0.514102744193224659,  0.803207531480644166, -0.671558954847017664, 
	},{
		 0.634393284163645488, -0.881921264348354939, -0.290284677254462442,  0.995184726672196929, 
		-0.098017140329560840, -0.956940335732208713,  0.471396736825997364,  0.773010453362736549, 
		-0.773010453362736993, -0.471396736825998308,  0.956940335732208935,  0.098017140329559285, 
		-0.995184726672197040,  0.290284677254462220,  0.881921264348354605, -0.634393284163644267, 
		-0.634393284163645932,  0.881921264348355272,  0.290284677254464329, -0.995184726672196818, 
		 0.098017140329560631,  0.956940335732208602, -0.471396736825999529, -0.773010453362734995, 
		 0.773010453362735106,  0.471396736825999307, -0.956940335732208602, -0.098017140329560382, 
		 0.995184726672196818, -0.290284677254464552, -0.881921264348353495,  0.634393284163643378, 
	},{
		 0.595699304492433357, -0.941544065183020806, -0.049067674327418029,  0.970031253194544085, 
		-0.514102744193221439, -0.671558954847018108,  0.903989293123443005,  0.146730474455361803, 
		-0.989176509964780903,  0.427555093430283306,  0.740951125354960105, -0.857728610000271674, 
		-0.242980179903263954,  0.998795456205172405, -0.336889853392221328, -0.803207531480645831, 
		 0.803207531480644388,  0.336889853392220162, -0.998795456205172294,  0.242980179903265148, 
		 0.857728610000272895, -0.740951125354960882, -0.427555093430282196,  0.989176509964780570, 
		-0.146730474455362997, -0.903989293123444004,  0.671558954847020440,  0.514102744193221883, 
		-0.970031253194543308,  0.049067674327419257,  0.941544065183021361, -0.595699304492435466, 
	},{
		 0.555570233019602289, -0.980785280403230431,  0.195090322016128304,  0.831469612302545014, 
		-0.831469612302545125, -0.195090322016128026,  0.980785280403230320, -0.555570233019601512, 
		-0.555570233019602622,  0.980785280403230431, -0.195090322016128581, -0.831469612302544903, 
		 0.831469612302545791,  0.195090322016126888, -0.980785280403230098,  0.555570233019601067, 
		 0.555570233019600179, -0.980785280403230986,  0.195090322016131440,  0.831469612302547123, 
		-0.831469612302543459, -0.195090322016130968,  0.980785280403230875, -0.555570233019600512, 
		-0.555570233019603621,  0.980785280403230209, -0.195090322016127360, -0.831469612302545569, 
		 0.831469612302545125,  0.195090322016128082, -0.980785280403230320,  0.555570233019602955, 
	},{
		 0.514102744193221661, -0.998795456205172405,  0.427555093430282140,  0.595699304492433246, 
		-0.989176509964781014,  0.336889853392220218,  0.671558954847018219, -0.970031253194544085, 
		 0.242980179903262428,  0.740951125354960105, -0.941544065183020251,  0.146730474455360332, 
		 0.803207531480645720, -0.903989293123442783,  0.049067674327416683,  0.857728610000272784, 
		-0.857728610000273228, -0.049067674327419250,  0.903989293123442339, -0.803207531480644166, 
		-0.146730474455359361,  0.941544065183021139, -0.740951125354960771, -0.242980179903264926, 
		 0.970031253194543419, -0.671558954847017664, -0.336889853392217609,  0.989176509964781125, 
		-0.595699304492435466, -0.427555093430282862,  0.998795456205172294, -0.514102744193227101, 
	},{
		 0.471396736825997587, -0.995184726672196929,  0.634393284163645599,  0.290284677254462553, 
		-0.956940335732208713,  0.773010453362736993,  0.098017140329560812, -0.881921264348354383, 
		 0.881921264348355494, -0.098017140329561242, -0.773010453362736771,  0.956940335732208824, 
		-0.290284677254462109, -0.634393284163645932,  0.995184726672196818, -0.471396736825999751, 
		-0.471396736825995866,  0.995184726672196707, -0.634393284163646598, -0.290284677254461276, 
		 0.956940335732208602, -0.773010453362737326, -0.098017140329560382,  0.881921264348355050, 
		-0.881921264348354828,  0.098017140329559896,  0.773010453362737548, -0.956940335732208491, 
		 0.290284677254460832,  0.634393284163647042, -0.995184726672197373,  0.471396736826001694, 
	},{
		 0.427555093430282196, -0.970031253194543974,  0.803207531480645054, -0.049067674327418424, 
		-0.740951125354958662,  0.989176509964781014, -0.514102744193221217, -0.336889853392219552, 
		 0.941544065183020806, -0.857728610000271674,  0.146730474455360332,  0.671558954847017331, 
		-0.998795456205172405,  0.595699304492433468,  0.242980179903260873, -0.903989293123443782, 
		 0.903989293123444115, -0.242980179903261595, -0.595699304492432913,  0.998795456205172183, 
		-0.671558954847017886, -0.146730474455359611,  0.857728610000273117, -0.941544065183021028, 
		 0.336889853392216942,  0.514102744193222105, -0.989176509964780570,  0.740951125354957996, 
		 0.049067674327416808, -0.803207531480646719,  0.970031253194545751, -0.427555093430284638, 
	},{
		 0.382683432365089837, -0.923879532511286850,  0.923879532511286850, -0.382683432365089893, 
		-0.382683432365090559,  0.923879532511286738, -0.923879532511287072,  0.382683432365089560, 
		 0.382683432365090892, -0.923879532511286294,  0.923879532511286850, -0.382683432365089227, 
		-0.382683432365091225,  0.923879532511286405, -0.923879532511285406,  0.382683432365092169, 
		 0.382683432365088283, -0.923879532511286516,  0.923879532511286627, -0.382683432365088505, 
		-0.382683432365091947,  0.923879532511288071, -0.923879532511287849,  0.382683432365091447, 
		 0.382683432365089005, -0.923879532511286850,  0.923879532511286294, -0.382683432365094389, 
		-0.382683432365092613,  0.923879532511285628, -0.923879532511284851,  0.382683432365090781, 
	},{
		 0.336889853392220051, -0.857728610000272118,  0.989176509964781014, -0.671558954847018885, 
		 0.049067674327417418,  0.595699304492433468, -0.970031253194543863,  0.903989293123442894, 
		-0.427555093430283195, -0.242980179903263954,  0.803207531480645720, -0.998795456205172405, 
		 0.740951125354958773, -0.146730474455363497, -0.514102744193224437,  0.941544065183021028, 
		-0.941544065183021250,  0.514102744193218775,  0.146730474455362997, -0.740951125354958440, 
		 0.998795456205172183, -0.803207531480643944,  0.242980179903264454,  0.427555093430279587, 
		-0.903989293123444226,  0.970031253194544085, -0.595699304492435244, -0.049067674327413380, 
		 0.671558954847013334, -0.989176509964781681,  0.857728610000270564, -0.336889853392219440, 
	},{
		 0.290284677254462331, -0.773010453362736882,  0.995184726672196818, -0.881921264348354828, 
		 0.471396736825997364,  0.098017140329560812, -0.634393284163645599,  0.956940335732209380, 
		-0.956940335732208935,  0.634393284163644378, -0.098017140329560992, -0.471396736825998752, 
		 0.881921264348354716, -0.995184726672196818,  0.773010453362735217, -0.290284677254465051, 
		-0.290284677254461276,  0.773010453362737326, -0.995184726672197151,  0.881921264348356604, 
		-0.471396736825999085, -0.098017140329560631,  0.634393284163646820, -0.956940335732209824, 
		 0.956940335732209490, -0.634393284163640381,  0.098017140329566488,  0.471396736825993867, 
		-0.881921264348353828,  0.995184726672196929, -0.773010453362736549,  0.290284677254460111, 
	},{
		 0.242980179903263982, -0.671558954847018330,  0.941544065183020695, -0.989176509964781014, 
		 0.803207531480644832, -0.427555093430281807, -0.049067674327418521,  0.514102744193220773, 
		-0.857728610000272562,  0.998795456205172405, -0.903989293123442783,  0.595699304492433468, 
		-0.146730474455363497, -0.336889853392216942,  0.740951125354960549, -0.970031253194544196, 
		 0.970031253194544196, -0.740951125354960771,  0.336889853392217165,  0.146730474455363247, 
		-0.595699304492433357,  0.903989293123442672, -0.998795456205172516,  0.857728610000270897, 
		-0.514102744193227101,  0.049067674327418764,  0.427555093430286415, -0.803207531480642611, 
		 0.989176509964781236, -0.941544065183023138,  0.671558954847019440, -0.242980179903259930, 
	},{
		 0.195090322016128331, -0.555570233019602178,  0.831469612302545014, -0.980785280403230320, 
		 0.980785280403230431, -0.831469612302545014,  0.555570233019601512, -0.195090322016128581, 
		-0.195090322016126777,  0.555570233019602844, -0.831469612302545014,  0.980785280403230098, 
		-0.980785280403230986,  0.831469612302545569, -0.555570233019600734,  0.195090322016131218, 
		 0.195090322016127610, -0.555570233019603621,  0.831469612302543459, -0.980785280403230320, 
		 0.980785280403230098, -0.831469612302547012,  0.555570233019602955, -0.195090322016126888, 
		-0.195090322016124973,  0.555570233019607285, -0.831469612302545902,  0.980785280403229764, 
		-0.980785280403229320,  0.831469612302544570, -0.555570233019605175,  0.195090322016122558, 
	},{
		 0.146730474455361748, -0.427555093430282474,  0.671558954847018108, -0.857728610000272340, 
		 0.970031253194543863, -0.998795456205172405,  0.941544065183020362, -0.803207531480644610, 
		 0.595699304492433690, -0.336889853392221328,  0.049067674327416683,  0.242980179903260873, 
		-0.514102744193224437,  0.740951125354960549, -0.903989293123443893,  0.989176509964781014, 
		-0.989176509964781014,  0.903989293123443893, -0.740951125354960549,  0.514102744193224437, 
		-0.242980179903260873, -0.049067674327420228,  0.336889853392221328, -0.595699304492433690, 
		 0.803207531480644610, -0.941544065183020362,  0.998795456205172294, -0.970031253194544751, 
		 0.857728610000274116, -0.671558954847021994,  0.427555093430287303, -0.146730474455368354, 
	},{
		 0.098017140329560548, -0.290284677254462442,  0.471396736825997975, -0.634393284163645377, 
		 0.773010453362736549, -0.881921264348354383,  0.956940335732209380, -0.995184726672197040, 
		 0.995184726672196818, -0.956940335732208824,  0.881921264348355272, -0.773010453362737548, 
		 0.634393284163643934, -0.471396736825999529,  0.290284677254461498, -0.098017140329563809, 
		-0.098017140329560382,  0.290284677254465051, -0.471396736825996476,  0.634393284163646820, 
		-0.773010453362735439,  0.881921264348355272, -0.956940335732207825,  0.995184726672196818, 
		-0.995184726672196596,  0.956940335732207270, -0.881921264348357714,  0.773010453362738659, 
		-0.634393284163645266,  0.471396736825994755, -0.290284677254456336,  0.098017140329565516, 
	},{
		 0.049067674327417904, -0.146730474455361443,  0.242980179903264232, -0.336889853392219329, 
		 0.427555093430282807, -0.514102744193220662,  0.595699304492432247, -0.671558954847017220, 
		 0.740951125354960216, -0.803207531480645831,  0.857728610000272784, -0.903989293123443782, 
		 0.941544065183021028, -0.970031253194544196,  0.989176509964781014, -0.998795456205172405, 
		 0.998795456205172405, -0.989176509964781014,  0.970031253194544085, -0.941544065183021028, 
		 0.903989293123443671, -0.857728610000272562,  0.803207531480641390, -0.740951125354960105, 
		 0.671558954847014444, -0.595699304492434911,  0.514102744193217442, -0.427555093430284194, 
		 0.336889853392215777, -0.242980179903266591,  0.146730474455357668, -0.049067674327421214, 
	},{
		 0.000000000000000061, -0.000000000000000184,  0.000000000000000306, -0.000000000000000429, 
		 0.000000000000000551,  0.000000000000001103, -0.000000000000000980,  0.000000000000000858, 
		-0.000000000000000735,  0.000000000000000613, -0.000000000000000491, -0.000000000000003185, 
		-0.000000000000000246, -0.000000000000003430, -0.000000000000000001,  0.000000000000003431, 
		 0.000000000000000244,  0.000000000000003186,  0.000000000000000489,  0.000000000000002941, 
		-0.000000000000006371, -0.000000000000004409,  0.000000000000000979,  0.000000000000002451, 
		-0.000000000000005881, -0.000000000000004899,  0.000000000000001469,  0.000000000000001961, 
		-0.000000000000005392, -0.000000000000005389,  0.000000000000001959,  0.000000000000001472, 
	},{
		-0.049067674327418008,  0.146730474455361942, -0.242980179903264509,  0.336889853392220218, 
		-0.427555093430281807,  0.514102744193222660, -0.595699304492433801,  0.671558954847018441, 
		-0.740951125354958884,  0.803207531480644388, -0.857728610000273228,  0.903989293123444115, 
		-0.941544065183021250,  0.970031253194544196, -0.989176509964781014,  0.998795456205172405, 
		-0.998795456205172405,  0.989176509964781125, -0.970031253194544307,  0.941544065183021361, 
		-0.903989293123441229,  0.857728610000269787, -0.803207531480642611,  0.740951125354956774, 
		-0.671558954847016221,  0.595699304492431358, -0.514102744193219996,  0.427555093430280642, 
		-0.336889853392218996,  0.242980179903263260, -0.146730474455361554,  0.049067674327418272, 
	},{
		-0.098017140329560645,  0.290284677254462054, -0.471396736825997476,  0.634393284163646043, 
		-0.773010453362736993,  0.881921264348355494, -0.956940335732208935,  0.995184726672196818, 
		-0.995184726672197040,  0.956940335732208491, -0.881921264348356493,  0.773010453362737104, 
		-0.634393284163643600,  0.471396736825999307, -0.290284677254461498,  0.098017140329564045, 
		 0.098017140329559896, -0.290284677254464329,  0.471396736825995644, -0.634393284163640381, 
		 0.773010453362738992, -0.881921264348354494,  0.956940335732207270, -0.995184726672197262, 
		 0.995184726672196929, -0.956940335732210157,  0.881921264348352385, -0.773010453362736216, 
		 0.634393284163647930, -0.471396736825991758,  0.290284677254460111, -0.098017140329562588, 
	},{
		-0.146730474455361859,  0.427555093430282140, -0.671558954847018885,  0.857728610000271896, 
		-0.970031253194544085,  0.998795456205172405, -0.941544065183020917,  0.803207531480645720, 
		-0.595699304492432469,  0.336889853392220162, -0.049067674327419250, -0.242980179903261595, 
		 0.514102744193218775, -0.740951125354960771,  0.903989293123443893, -0.989176509964781014, 
		 0.989176509964781125, -0.903989293123444226,  0.740951125354961215, -0.514102744193225436, 
		 0.242980179903262289,  0.049067674327411423, -0.336889853392219440,  0.595699304492437687, 
		-0.803207531480643166,  0.941544065183021806, -0.998795456205172183,  0.970031253194543752, 
		-0.857728610000268565,  0.671558954847019440, -0.427555093430278033,  0.146730474455365439, 
	},{
		-0.195090322016128193,  0.555570233019601845, -0.831469612302545125,  0.980785280403230431, 
		-0.980785280403230320,  0.831469612302544792, -0.555570233019602733,  0.195090322016126888, 
		 0.195090322016128220, -0.555570233019600956,  0.831469612302545569, -0.980785280403230986, 
		 0.980785280403230875, -0.831469612302545458,  0.555570233019600734, -0.195090322016131440, 
		-0.195090322016127138,  0.555570233019602955, -0.831469612302542904,  0.980785280403230098, 
		-0.980785280403230431,  0.831469612302544014, -0.555570233019598736,  0.195090322016122086, 
		 0.195090322016122558, -0.555570233019599069,  0.831469612302544348, -0.980785280403230542, 
		 0.980785280403229986, -0.831469612302542682,  0.555570233019596627, -0.195090322016133605, 
	},{
		-0.242980179903263871,  0.671558954847018330, -0.941544065183021028,  0.989176509964780903, 
		-0.803207531480645498,  0.427555093430281419,  0.049067674327416926, -0.514102744193222327, 
		 0.857728610000271452, -0.998795456205172294,  0.903989293123442339, -0.595699304492432913, 
		 0.146730474455362997,  0.336889853392217165, -0.740951125354960549,  0.970031253194544085, 
		-0.970031253194544307,  0.740951125354961215, -0.336889853392218053, -0.146730474455362053, 
		 0.595699304492432136, -0.903989293123441895,  0.998795456205172627, -0.857728610000275671, 
		 0.514102744193216998, -0.049067674327414358, -0.427555093430283750,  0.803207531480644943, 
		-0.989176509964780681,  0.941544065183022028, -0.671558954847022327,  0.242980179903257071, 
	},{
		-0.290284677254462387,  0.773010453362737215, -0.995184726672196929,  0.881921264348355272, 
		-0.471396736825998308, -0.098017140329561242,  0.634393284163644378, -0.956940335732208824, 
		 0.956940335732208491, -0.634393284163643378,  0.098017140329563560,  0.471396736825996254, 
		-0.881921264348354939,  0.995184726672196818, -0.773010453362735217,  0.290284677254465273, 
		 0.290284677254460832, -0.773010453362736771,  0.995184726672196263, -0.881921264348353828, 
		 0.471396736826000362,  0.098017140329566002, -0.634393284163645266,  0.956940335732207159, 
		-0.956940335732208158,  0.634393284163647930, -0.098017140329555261, -0.471396736825997364, 
		 0.881921264348352163, -0.995184726672196596,  0.773010453362738992, -0.290284677254457280, 
	},{
		-0.336889853392220162,  0.857728610000272007, -0.989176509964781125,  0.671558954847018219, 
		-0.049067674327418521, -0.595699304492433801,  0.970031253194544418, -0.903989293123443671, 
		 0.427555093430285082,  0.242980179903265148, -0.803207531480644166,  0.998795456205172183, 
		-0.740951125354958440,  0.146730474455363247,  0.514102744193224437, -0.941544065183021028, 
		 0.941544065183021361, -0.514102744193225436, -0.146730474455362053,  0.740951125354962437, 
		-0.998795456205172183,  0.803207531480644943, -0.242980179903259458, -0.427555093430277588, 
		 0.903989293123443116, -0.970031253194542975,  0.595699304492437687,  0.049067674327417300, 
		-0.671558954847021328,  0.989176509964780126, -0.857728610000272562,  0.336889853392216665, 
	},{
		-0.382683432365089726,  0.923879532511286850, -0.923879532511286738,  0.382683432365089060, 
		 0.382683432365089560, -0.923879532511286960,  0.923879532511286294, -0.382683432365091225, 
		-0.382683432365089005,  0.923879532511286738, -0.923879532511286516,  0.382683432365088505, 
		 0.382683432365091669, -0.923879532511287849,  0.923879532511285406, -0.382683432365092391, 
		-0.382683432365094389,  0.923879532511288959, -0.923879532511284296,  0.382683432365096221, 
		 0.382683432365084009, -0.923879532511284629,  0.923879532511288626, -0.382683432365093501, 
		-0.382683432365086729,  0.923879532511285739, -0.923879532511287405,  0.382683432365090781, 
		 0.382683432365089449, -0.923879532511286850,  0.923879532511286294, -0.382683432365088061, 
	},{
		-0.427555093430282251,  0.970031253194543974, -0.803207531480645387,  0.049067674327418397, 
		 0.740951125354959106, -0.989176509964780903,  0.514102744193220995,  0.336889853392221217, 
		-0.941544065183021361,  0.857728610000272895, -0.146730474455359361, -0.671558954847017886, 
		 0.998795456205172183, -0.595699304492433357, -0.242980179903260873,  0.903989293123443671, 
		-0.903989293123441229,  0.242980179903262289,  0.595699304492432136, -0.998795456205172183, 
		 0.671558954847013667,  0.146730474455364940, -0.857728610000272118,  0.941544065183021806, 
		-0.336889853392225935, -0.514102744193225880,  0.989176509964781236, -0.740951125354960105, 
		-0.049067674327413380,  0.803207531480648718, -0.970031253194543197,  0.427555093430281974, 
	},{
		-0.471396736825997698,  0.995184726672196929, -0.634393284163645377, -0.290284677254462553, 
		 0.956940335732208935, -0.773010453362736771, -0.098017140329560992,  0.881921264348355272, 
		-0.881921264348356493,  0.098017140329563560,  0.773010453362735106, -0.956940335732209713, 
		 0.290284677254465051,  0.634393284163643378, -0.995184726672197151,  0.471396736825999918, 
		 0.471396736826001694, -0.995184726672196596,  0.634393284163641824,  0.290284677254460111, 
		-0.956940335732210268,  0.773010453362738326,  0.098017140329565516, -0.881921264348354050, 
		 0.881921264348352607, -0.098017140329562588, -0.773010453362740213,  0.956940335732209380, 
		-0.290284677254457280, -0.634393284163644156,  0.995184726672196374, -0.471396736825986540, 
	},{
		-0.514102744193221661,  0.998795456205172405, -0.427555093430282696, -0.595699304492434023, 
		 0.989176509964780903, -0.336889853392219829, -0.671558954847018441,  0.970031253194544085, 
		-0.242980179903264426, -0.740951125354960882,  0.941544065183021139, -0.146730474455359611, 
		-0.803207531480643944,  0.903989293123442672, -0.049067674327420228, -0.857728610000272562, 
		 0.857728610000269787,  0.049067674327411423, -0.903989293123441895,  0.803207531480644943, 
		 0.146730474455364940, -0.941544065183023027,  0.740951125354962104,  0.242980179903262788, 
		-0.970031253194544529,  0.671558954847014444,  0.336889853392214833, -0.989176509964780681, 
		 0.595699304492432469,  0.427555093430285971, -0.998795456205172738,  0.514102744193224548, 
	},{
		-0.555570233019602289,  0.980785280403230431, -0.195090322016128026, -0.831469612302545014, 
		 0.831469612302544792,  0.195090322016128442, -0.980785280403230320,  0.555570233019600179, 
		 0.555570233019603843, -0.980785280403230209,  0.195090322016127610,  0.831469612302545236, 
		-0.831469612302545569, -0.195090322016127138,  0.980785280403230098, -0.555570233019598292, 
		-0.555570233019605619,  0.980785280403229764, -0.195090322016125445, -0.831469612302546457, 
		 0.831469612302544348,  0.195090322016129303, -0.980785280403230542,  0.555570233019602400, 
		 0.555570233019601512, -0.980785280403230764,  0.195090322016130246,  0.831469612302543792, 
		-0.831469612302547012, -0.195090322016124473,  0.980785280403232318, -0.555570233019594628, 
	},{
		-0.595699304492433357,  0.941544065183020695,  0.049067674327417418, -0.970031253194544085, 
		 0.514102744193220773,  0.671558954847018441, -0.903989293123443671, -0.146730474455363497, 
		 0.989176509964780570, -0.427555093430282196, -0.740951125354960771,  0.857728610000273117, 
		 0.242980179903264454, -0.998795456205172516,  0.336889853392221328,  0.803207531480641390, 
		-0.803207531480642611, -0.336889853392219440,  0.998795456205172627, -0.242980179903259458, 
		-0.857728610000272118,  0.740951125354962104,  0.427555093430286859, -0.989176509964780903, 
		 0.146730474455365439,  0.903989293123445892, -0.671558954847017331, -0.514102744193219108, 
		 0.970031253194542420, -0.049067674327415822, -0.941544065183024803,  0.595699304492438797, 
	},{
		-0.634393284163645377,  0.881921264348355050,  0.290284677254462664, -0.995184726672196818, 
		 0.098017140329559285,  0.956940335732208824, -0.471396736825998752, -0.773010453362737548, 
		 0.773010453362737104,  0.471396736825996254, -0.956940335732209713, -0.098017140329563560, 
		 0.995184726672197040, -0.290284677254461998, -0.881921264348357936,  0.634393284163647153, 
		 0.634393284163648263, -0.881921264348357270, -0.290284677254463386,  0.995184726672196263, 
		-0.098017140329562089, -0.956940335732210157,  0.471396736826001250,  0.773010453362738104, 
		-0.773010453362741212, -0.471396736825996920,  0.956940335732207381,  0.098017140329557217, 
		-0.995184726672197151,  0.290284677254468104,  0.881921264348361711, -0.634393284163641158, 
	},{
		-0.671558954847018441,  0.803207531480644721,  0.514102744193221328, -0.903989293123443449, 
		-0.336889853392221550,  0.970031253194543974,  0.146730474455360110, -0.998795456205172405, 
		 0.049067674327415829,  0.989176509964780570, -0.242980179903264926, -0.941544065183021028, 
		 0.427555093430279587,  0.857728610000270897, -0.595699304492433690, -0.740951125354960105, 
		 0.740951125354956774,  0.595699304492437687, -0.857728610000275671, -0.427555093430277588, 
		 0.941544065183021806,  0.242980179903262788, -0.989176509964780903, -0.049067674327420721, 
		 0.998795456205172627, -0.146730474455355253, -0.970031253194542642,  0.336889853392223604, 
		 0.903989293123448667, -0.514102744193233874, -0.803207531480637615,  0.671558954847025991, 
	},{
		-0.707106781186547462,  0.707106781186547684,  0.707106781186547795, -0.707106781186547351, 
		-0.707106781186546352,  0.707106781186547573,  0.707106781186548572, -0.707106781186547906, 
		-0.707106781186545796,  0.707106781186545574,  0.707106781186548017, -0.707106781186548350, 
		-0.707106781186545241,  0.707106781186546129,  0.707106781186542466, -0.707106781186543909, 
		-0.707106781186549793,  0.707106781186546685,  0.707106781186547018, -0.707106781186549460, 
		-0.707106781186544242,  0.707106781186552236,  0.707106781186551458, -0.707106781186544908, 
		-0.707106781186548683,  0.707106781186547684,  0.707106781186546018, -0.707106781186540467, 
		-0.707106781186543243,  0.707106781186543243,  0.707106781186540467, -0.707106781186546018, 
	},{
		-0.740951125354959217,  0.595699304492433246,  0.857728610000271896, -0.427555093430281308, 
		-0.941544065183020251,  0.242980179903264093,  0.989176509964781125, -0.049067674327419250, 
		-0.998795456205172405, -0.146730474455362997,  0.970031253194543419,  0.336889853392216942, 
		-0.903989293123444226, -0.514102744193227101,  0.803207531480644610,  0.671558954847014444, 
		-0.671558954847016221, -0.803207531480643166,  0.514102744193216998,  0.903989293123443116, 
		-0.336889853392225935, -0.970031253194544529,  0.146730474455365439,  0.998795456205172627, 
		 0.049067674327416808, -0.989176509964782014, -0.242980179903265148,  0.941544065183017476, 
		 0.427555093430285527, -0.857728610000273117, -0.595699304492427029,  0.740951125354968321, 
	},{
		-0.773010453362736993,  0.471396736825997975,  0.956940335732209046, -0.098017140329559174, 
		-0.995184726672197040, -0.290284677254462109,  0.881921264348354716,  0.634393284163643934, 
		-0.634393284163643600, -0.881921264348354939,  0.290284677254465051,  0.995184726672197040, 
		 0.098017140329559660, -0.956940335732207825, -0.471396736825992146,  0.773010453362738104, 
		 0.773010453362738659, -0.471396736826003859, -0.956940335732208158,  0.098017140329558675, 
		 0.995184726672196263,  0.290284677254459167, -0.881921264348354494, -0.634393284163649818, 
		 0.634393284163648707,  0.881921264348355161, -0.290284677254457779, -0.995184726672197817, 
		-0.098017140329574287,  0.956940335732211822,  0.471396736825992591, -0.773010453362737771, 
	},{
		-0.803207531480644832,  0.336889853392220051,  0.998795456205172405,  0.242980179903262428, 
		-0.857728610000272562, -0.740951125354958884,  0.427555093430285082,  0.989176509964780570, 
		 0.146730474455359611, -0.903989293123444004, -0.671558954847017664,  0.514102744193222105, 
		 0.970031253194544085,  0.049067674327418764, -0.941544065183020362, -0.595699304492434911, 
		 0.595699304492431358,  0.941544065183021806, -0.049067674327414358, -0.970031253194542975, 
		-0.514102744193225880,  0.671558954847014444,  0.903989293123445892, -0.146730474455355253, 
		-0.989176509964782014, -0.427555093430276256,  0.740951125354963103,  0.857728610000276670, 
		-0.242980179903268478, -0.998795456205171850, -0.336889853392216720,  0.803207531480638170, 
	},{
		-0.831469612302545347,  0.195090322016127915,  0.980785280403230320,  0.555570233019601512, 
		-0.555570233019602733, -0.980785280403230320, -0.195090322016128082,  0.831469612302547123, 
		 0.831469612302545458, -0.195090322016131218, -0.980785280403230320, -0.555570233019600179, 
		 0.555570233019601178,  0.980785280403230098,  0.195090322016123030, -0.831469612302542127, 
		-0.831469612302546457,  0.195090322016129275,  0.980785280403231319,  0.555570233019607729, 
		-0.555570233019599513, -0.980785280403230431, -0.195090322016124973,  0.831469612302548899, 
		 0.831469612302547567, -0.195090322016141293, -0.980785280403228099, -0.555570233019609283, 
		 0.555570233019597848,  0.980785280403230875,  0.195090322016126888, -0.831469612302547900, 
	},{
		-0.857728610000272007,  0.049067674327418154,  0.903989293123443449,  0.803207531480644721, 
		-0.146730474455362164, -0.941544065183020917, -0.740951125354961104,  0.242980179903261123, 
		 0.970031253194543308,  0.671558954847020440, -0.336889853392217609, -0.989176509964780570, 
		-0.595699304492435244,  0.427555093430286415,  0.998795456205172294,  0.514102744193217442, 
		-0.514102744193219996, -0.998795456205172183, -0.427555093430283750,  0.595699304492437687, 
		 0.989176509964781236,  0.336889853392214833, -0.671558954847017331, -0.970031253194542642, 
		-0.242980179903265148,  0.740951125354963103,  0.941544065183021139,  0.146730474455369797, 
		-0.803207531480652825, -0.903989293123440674, -0.049067674327418764,  0.857728610000268121, 
	},{
		-0.881921264348354939, -0.098017140329560840,  0.773010453362736549,  0.956940335732208935, 
		 0.290284677254462220, -0.634393284163645932, -0.995184726672196818, -0.471396736825999529, 
		 0.471396736825999307,  0.995184726672196818,  0.634393284163643378, -0.290284677254461998, 
		-0.956940335732207825, -0.773010453362741212,  0.098017140329565017,  0.881921264348355605, 
		 0.881921264348355827,  0.098017140329565516, -0.773010453362740879, -0.956940335732207936, 
		-0.290284677254462442,  0.634393284163643045,  0.995184726672197484,  0.471396736825993479, 
		-0.471396736825986540, -0.995184726672196707, -0.634393284163638049,  0.290284677254454948, 
		 0.956940335732209824,  0.773010453362745875, -0.098017140329557703, -0.881921264348358935, 
	},{
		-0.903989293123443338, -0.242980179903264509,  0.595699304492433468,  0.998795456205172405, 
		 0.671558954847018441, -0.146730474455362414, -0.857728610000271008, -0.941544065183021250, 
		-0.336889853392220606,  0.514102744193221883,  0.989176509964781125,  0.740951125354957996, 
		-0.049067674327413380, -0.803207531480642611, -0.970031253194544751, -0.427555093430284194, 
		 0.427555093430280642,  0.970031253194543752,  0.803207531480644943,  0.049067674327417300, 
		-0.740951125354960105, -0.989176509964780681, -0.514102744193219108,  0.336889853392223604, 
		 0.941544065183017476,  0.857728610000276670,  0.146730474455369797, -0.671558954847012890, 
		-0.998795456205172738, -0.595699304492438020,  0.242980179903258958,  0.903989293123441451, 
	},{
		-0.923879532511286738, -0.382683432365089893,  0.382683432365089060,  0.923879532511286183, 
		 0.923879532511286850,  0.382683432365089116, -0.382683432365091447, -0.923879532511286516, 
		-0.923879532511287960, -0.382683432365088283,  0.382683432365089005,  0.923879532511285517, 
		 0.923879532511288959,  0.382683432365090781, -0.382683432365093057, -0.923879532511284407, 
		-0.923879532511287294, -0.382683432365086729,  0.382683432365084009,  0.923879532511286183, 
		 0.923879532511285628,  0.382683432365095777, -0.382683432365088061, -0.923879532511287849, 
		-0.923879532511289292, -0.382683432365078569,  0.382683432365092169,  0.923879532511284074, 
		 0.923879532511282187,  0.382683432365087617, -0.382683432365083065, -0.923879532511291179, 
	},{
		-0.941544065183020806, -0.514102744193221439,  0.146730474455361803,  0.740951125354960105, 
		 0.998795456205172405,  0.803207531480644388,  0.242980179903265148, -0.427555093430282196, 
		-0.903989293123444004, -0.970031253194543308, -0.595699304492435466,  0.049067674327416808, 
		 0.671558954847013334,  0.989176509964781236,  0.857728610000274116,  0.336889853392215777, 
		-0.336889853392218996, -0.857728610000268565, -0.989176509964780681, -0.671558954847021328, 
		-0.049067674327413380,  0.595699304492432469,  0.970031253194542420,  0.903989293123448667, 
		 0.427555093430285527, -0.242980179903268478, -0.803207531480652825, -0.998795456205172738, 
		-0.740951125354957774, -0.146730474455351367,  0.514102744193213668,  0.941544065183020473, 
	},{
		-0.956940335732208824, -0.634393284163644822, -0.098017140329561492,  0.471396736825998419, 
		 0.881921264348354605,  0.995184726672196818,  0.773010453362735217,  0.290284677254461498, 
		-0.290284677254461498, -0.773010453362735217, -0.995184726672197151, -0.881921264348357936, 
		-0.471396736825992146,  0.098017140329565017,  0.634393284163647597,  0.956940335732209157, 
		 0.956940335732209157,  0.634393284163647597,  0.098017140329565017, -0.471396736825992146, 
		-0.881921264348357936, -0.995184726672196485, -0.773010453362735217, -0.290284677254461498, 
		 0.290284677254461498,  0.773010453362735217,  0.995184726672196485,  0.881921264348357936, 
		 0.471396736826004692, -0.098017140329550875, -0.634393284163636606, -0.956940335732205050, 
	},{
		-0.970031253194543974, -0.740951125354959328, -0.336889853392220107,  0.146730474455362025, 
		 0.595699304492432469,  0.903989293123443782,  0.998795456205172405,  0.857728610000271230, 
		 0.514102744193224659,  0.049067674327419257, -0.427555093430282862, -0.803207531480646719, 
		-0.989176509964781681, -0.941544065183023138, -0.671558954847021994, -0.242980179903266591, 
		 0.242980179903263260,  0.671558954847019440,  0.941544065183022028,  0.989176509964780126, 
		 0.803207531480648718,  0.427555093430285971, -0.049067674327415822, -0.514102744193233874, 
		-0.857728610000273117, -0.998795456205171850, -0.903989293123440674, -0.595699304492438020, 
		-0.146730474455351367,  0.336889853392218552,  0.740951125354949891,  0.970031253194544640, 
	},{
		-0.980785280403230431, -0.831469612302545125, -0.555570233019601512, -0.195090322016128581, 
		 0.195090322016126888,  0.555570233019600179,  0.831469612302547123,  0.980785280403230875, 
		 0.980785280403230209,  0.831469612302545125,  0.555570233019602955,  0.195090322016130246, 
		-0.195090322016132162, -0.555570233019598736, -0.831469612302546235, -0.980785280403229209, 
		-0.980785280403230542, -0.831469612302542127, -0.555570233019604398, -0.195090322016124973, 
		 0.195090322016123530,  0.555570233019603177,  0.831469612302541350,  0.980785280403230209, 
		 0.980785280403229431,  0.831469612302539129,  0.555570233019611726,  0.195090322016133605, 
		-0.195090322016128803, -0.555570233019607618, -0.831469612302552230, -0.980785280403228543, 
	},{
		-0.989176509964781014, -0.903989293123443116, -0.740951125354959217, -0.514102744193222549, 
		-0.242980179903265509,  0.049067674327415579,  0.336889853392220384,  0.595699304492435910, 
		 0.803207531480644166,  0.941544065183021361,  0.998795456205172294,  0.970031253194545751, 
		 0.857728610000270564,  0.671558954847019440,  0.427555093430287303,  0.146730474455357668, 
		-0.146730474455361554, -0.427555093430278033, -0.671558954847022327, -0.857728610000272562, 
		-0.970031253194543197, -0.998795456205172738, -0.941544065183024803, -0.803207531480637615, 
		-0.595699304492427029, -0.336889853392216720, -0.049067674327418764,  0.242980179903258958, 
		 0.514102744193213668,  0.740951125354949891,  0.903989293123447779,  0.989176509964781903, 
	},{
		-0.995184726672196929, -0.956940335732209046, -0.881921264348354716, -0.773010453362737882, 
		-0.634393284163644267, -0.471396736825999751, -0.290284677254465051, -0.098017140329563809, 
		 0.098017140329564045,  0.290284677254465273,  0.471396736825999918,  0.634393284163647153, 
		 0.773010453362738104,  0.881921264348355605,  0.956940335732209157,  0.995184726672196929, 
		 0.995184726672196929,  0.956940335732208935,  0.881921264348355383,  0.773010453362737771, 
		 0.634393284163646820,  0.471396736825999529,  0.290284677254451229,  0.098017140329563560, 
		-0.098017140329571359, -0.290284677254458723, -0.471396736826006413, -0.634393284163641824, 
		-0.773010453362742767, -0.881921264348352385, -0.956940335732211267, -0.995184726672196263, 
	},{
		-0.998795456205172405, -0.989176509964781014, -0.970031253194544085, -0.941544065183020251, 
		-0.903989293123442783, -0.857728610000273228, -0.803207531480644166, -0.740951125354960771, 
		-0.671558954847017664, -0.595699304492435466, -0.514102744193227101, -0.427555093430284638, 
		-0.336889853392219440, -0.242980179903259930, -0.146730474455368354, -0.049067674327421214, 
		 0.049067674327418272,  0.146730474455365439,  0.242980179903257071,  0.336889853392216665, 
		 0.427555093430281974,  0.514102744193224548,  0.595699304492438797,  0.671558954847025991, 
		 0.740951125354968321,  0.803207531480638170,  0.857728610000268121,  0.903989293123441451, 
		 0.941544065183020473,  0.970031253194544640,  0.989176509964781903,  0.998795456205172849, 
	},{
		-1.000000000000000000, -1.000000000000000000, -1.000000000000000000, -1.000000000000000000, 
		-1.000000000000000000, -1.000000000000000000, -1.000000000000000000, -1.000000000000000000, 
		-1.000000000000000000, -1.000000000000000000, -1.000000000000000000, -1.000000000000000000, 
		-1.000000000000000000, -1.000000000000000000, -1.000000000000000000, -1.000000000000000000, 
		-1.000000000000000000, -1.000000000000000000, -1.000000000000000000, -1.000000000000000000, 
		-1.000000000000000000, -1.000000000000000000, -1.000000000000000000, -1.000000000000000000, 
		-1.000000000000000000, -1.000000000000000000, -1.000000000000000000, -1.000000000000000000, 
		-1.000000000000000000, -1.000000000000000000, -1.000000000000000000, -1.000000000000000000, 
	},{
		-0.998795456205172405, -0.989176509964781125, -0.970031253194543863, -0.941544065183020917, 
		-0.903989293123443671, -0.857728610000271008, -0.803207531480646053, -0.740951125354958440, 
		-0.671558954847020773, -0.595699304492433468, -0.514102744193225436, -0.427555093430283306, 
		-0.336889853392218552, -0.242980179903259458, -0.146730474455368326, -0.049067674327421699, 
		 0.049067674327417300,  0.146730474455363968,  0.242980179903268950,  0.336889853392214389, 
		 0.427555093430279365,  0.514102744193233874,  0.595699304492435688,  0.671558954847012224, 
		 0.740951125354964990,  0.803207531480643500,  0.857728610000265013,  0.903989293123444892, 
		 0.941544065183018142,  0.970031253194546306,  0.989176509964780681,  0.998795456205171739, 
	},{
		-0.995184726672196929, -0.956940335732208713, -0.881921264348354383, -0.773010453362736771, 
		-0.634393284163645932, -0.471396736825995866, -0.290284677254461276, -0.098017140329560382, 
		 0.098017140329559896,  0.290284677254460832,  0.471396736826001694,  0.634393284163648263, 
		 0.773010453362738659,  0.881921264348355827,  0.956940335732209157,  0.995184726672196929, 
		 0.995184726672196929,  0.956940335732209380,  0.881921264348356382,  0.773010453362739325, 
		 0.634393284163638049,  0.471396736825989982,  0.290284677254454948,  0.098017140329553804, 
		-0.098017140329566488, -0.290284677254467161, -0.471396736826001250, -0.634393284163647930, 
		-0.773010453362738326, -0.881921264348355605, -0.956940335732208935, -0.995184726672196818, 
	},{
		-0.989176509964781014, -0.903989293123443338, -0.740951125354959994, -0.514102744193220884, 
		-0.242980179903264204,  0.049067674327419986,  0.336889853392217387,  0.595699304492432913, 
		 0.803207531480645942,  0.941544065183019807,  0.998795456205172405,  0.970031253194545306, 
		 0.857728610000270120,  0.671558954847019107,  0.427555093430287303,  0.146730474455358167, 
		-0.146730474455360582, -0.427555093430276700, -0.671558954847020884, -0.857728610000278668, 
		-0.970031253194542531, -0.998795456205172183, -0.941544065183016476, -0.803207531480648718, 
		-0.595699304492430914, -0.336889853392208394, -0.049067674327424635,  0.242980179903266591, 
		 0.514102744193232208,  0.740951125354954443,  0.903989293123444448,  0.989176509964782680, 
	},{
		-0.980785280403230431, -0.831469612302545569, -0.555570233019602622, -0.195090322016126777, 
		 0.195090322016128220,  0.555570233019603843,  0.831469612302545458,  0.980785280403230209, 
		 0.980785280403230986,  0.831469612302543792,  0.555570233019607285,  0.195090322016128803, 
		-0.195090322016133133, -0.555570233019599069, -0.831469612302546235, -0.980785280403229098, 
		-0.980785280403230764, -0.831469612302542904, -0.555570233019606063, -0.195090322016141293, 
		 0.195090322016134576,  0.555570233019600290,  0.831469612302539129,  0.980785280403232207, 
		 0.980785280403230431,  0.831469612302550010,  0.555570233019592963,  0.195090322016125917, 
		-0.195090322016122086, -0.555570233019613391, -0.831469612302547900, -0.980785280403229653, 
	},{
		-0.970031253194543974, -0.740951125354958662, -0.336889853392219552,  0.146730474455360332, 
		 0.595699304492433468,  0.903989293123444115,  0.998795456205172183,  0.857728610000273117, 
		 0.514102744193222105,  0.049067674327416808, -0.427555093430284638, -0.803207531480647607, 
		-0.989176509964781903, -0.941544065183022916, -0.671558954847021994, -0.242980179903267063, 
		 0.242980179903262289,  0.671558954847018441,  0.941544065183021361,  0.989176509964782569, 
		 0.803207531480650494,  0.427555093430289079, -0.049067674327411916, -0.514102744193217887, 
		-0.857728610000270564, -0.998795456205172294, -0.903989293123443116, -0.595699304492431692, 
		-0.146730474455358167,  0.336889853392224992,  0.740951125354963658,  0.970031253194546084, 
	},{
		-0.956940335732208824, -0.634393284163645377, -0.098017140329559174,  0.471396736825996920, 
		 0.881921264348355272,  0.995184726672196707,  0.773010453362737326,  0.290284677254465051, 
		-0.290284677254464329, -0.773010453362736771, -0.995184726672196596, -0.881921264348357270, 
		-0.471396736826003859,  0.098017140329565516,  0.634393284163647597,  0.956940335732208935, 
		 0.956940335732209380,  0.634393284163648707,  0.098017140329566974, -0.471396736825989982, 
		-0.881921264348356604, -0.995184726672198150, -0.773010453362737771, -0.290284677254452117, 
		 0.290284677254456835,  0.773010453362740879,  0.995184726672195930,  0.881921264348354272, 
		 0.471396736825985707, -0.098017140329557703, -0.634393284163652482, -0.956940335732206715, 
	},{
		-0.941544065183020806, -0.514102744193221994,  0.146730474455360582,  0.740951125354958884, 
		 0.998795456205172405,  0.803207531480643944,  0.242980179903261345, -0.427555093430278921, 
		-0.903989293123442228, -0.970031253194544418, -0.595699304492428139,  0.049067674327418279, 
		 0.671558954847014000,  0.989176509964781236,  0.857728610000274116,  0.336889853392216221, 
		-0.336889853392218053, -0.857728610000275116, -0.989176509964781014, -0.671558954847023104, 
		-0.049067674327430505,  0.595699304492441128,  0.970031253194544862,  0.903989293123444448, 
		 0.427555093430289968, -0.242980179903277027, -0.803207531480649273, -0.998795456205172294, 
		-0.740951125354962437, -0.146730474455372711,  0.514102744193231320,  0.941544065183022472, 
	},{
		-0.923879532511286850, -0.382683432365090559,  0.382683432365089560,  0.923879532511286850, 
		 0.923879532511286405,  0.382683432365088283, -0.382683432365088505, -0.923879532511287849, 
		-0.923879532511286850, -0.382683432365092613,  0.382683432365090781,  0.923879532511288737, 
		 0.923879532511288626,  0.382683432365090337, -0.382683432365093057, -0.923879532511284296, 
		-0.923879532511287627, -0.382683432365088061,  0.382683432365082177,  0.923879532511285184, 
		 0.923879532511286738,  0.382683432365085785, -0.382683432365097609, -0.923879532511291623, 
		-0.923879532511291179, -0.382683432365096665,  0.382683432365086729,  0.923879532511287072, 
		 0.923879532511284851,  0.382683432365081289, -0.382683432365102105, -0.923879532511282631, 
	},{
		-0.903989293123443227, -0.242980179903263482,  0.595699304492433912,  0.998795456205172405, 
		 0.671558954847017442, -0.146730474455363247, -0.857728610000273006, -0.941544065183020029, 
		-0.336889853392217831,  0.514102744193217887,  0.989176509964780348,  0.740951125354961770, 
		-0.049067674327414358, -0.803207531480642944, -0.970031253194544751, -0.427555093430284638, 
		 0.427555093430279809,  0.970031253194543419,  0.803207531480637615,  0.049067674327419743, 
		-0.740951125354967655, -0.989176509964781125, -0.514102744193210337,  0.336889853392219440, 
		 0.941544065183025469,  0.857728610000272118,  0.146730474455375598, -0.671558954847018774, 
		-0.998795456205173071, -0.595699304492432469,  0.242980179903251381,  0.903989293123444004, 
	},{
		-0.881921264348355050, -0.098017140329560687,  0.773010453362737993,  0.956940335732208380, 
		 0.290284677254464329, -0.634393284163646598, -0.995184726672197151, -0.471396736825996476, 
		 0.471396736825995644,  0.995184726672196263,  0.634393284163641824, -0.290284677254463386, 
		-0.956940335732208158, -0.773010453362740879,  0.098017140329565017,  0.881921264348355383, 
		 0.881921264348356382,  0.098017140329566974, -0.773010453362739658, -0.956940335732208713, 
		-0.290284677254465273,  0.634393284163640381,  0.995184726672197928,  0.471396736826009854, 
		-0.471396736826007301, -0.995184726672197595, -0.634393284163642601,  0.290284677254462442, 
		 0.956940335732207825,  0.773010453362741434, -0.098017140329549904, -0.881921264348361711, 
	},{
		-0.857728610000272118,  0.049067674327417418,  0.903989293123442894,  0.803207531480645720, 
		-0.146730474455363497, -0.941544065183021250, -0.740951125354958440,  0.242980179903264454, 
		 0.970031253194544085,  0.671558954847013334, -0.336889853392219440, -0.989176509964781903, 
		-0.595699304492434467,  0.427555093430286859,  0.998795456205172294,  0.514102744193217887, 
		-0.514102744193219108, -0.998795456205172960, -0.427555093430285527,  0.595699304492435688, 
		 0.989176509964779571,  0.336889853392231431, -0.671558954847014444, -0.970031253194543641, 
		-0.242980179903256127,  0.740951125354949891,  0.941544065183023138,  0.146730474455362025, 
		-0.803207531480648718, -0.903989293123437676, -0.049067674327426591,  0.857728610000271119, 
	},{
		-0.831469612302545236,  0.195090322016128942,  0.980785280403230431,  0.555570233019602844, 
		-0.555570233019600956, -0.980785280403230209, -0.195090322016131218,  0.831469612302545125, 
		 0.831469612302543792, -0.195090322016133605, -0.980785280403229320, -0.555570233019604842, 
		 0.555570233019601956,  0.980785280403229986,  0.195090322016123030, -0.831469612302541905, 
		-0.831469612302547012,  0.195090322016127860,  0.980785280403228099,  0.555570233019597848, 
		-0.555570233019597071, -0.980785280403228321, -0.195090322016128803,  0.831469612302538574, 
		 0.831469612302542460, -0.195090322016122086, -0.980785280403232540, -0.555570233019602733, 
		 0.555570233019592186,  0.980785280403229431,  0.195090322016134576, -0.831469612302551120, 
	},{
		-0.803207531480644943,  0.336889853392220218,  0.998795456205172405,  0.242980179903264093, 
		-0.857728610000273228, -0.740951125354958329,  0.427555093430282196,  0.989176509964781125, 
		 0.146730474455356696, -0.903989293123445004, -0.671558954847016221,  0.514102744193223327, 
		 0.970031253194543752,  0.049067674327418272, -0.941544065183020362, -0.595699304492435244, 
		 0.595699304492430581,  0.941544065183017476, -0.049067674327426598, -0.970031253194545862, 
		-0.514102744193216221,  0.671558954847022438,  0.903989293123441451, -0.146730474455364940, 
		-0.989176509964781236, -0.427555093430281086,  0.740951125354959106,  0.857728610000272562, 
		-0.242980179903261817, -0.998795456205172183, -0.336889853392224048,  0.803207531480641723, 
	},{
		-0.773010453362736882,  0.471396736825997364,  0.956940335732209380, -0.098017140329560992, 
		-0.995184726672196818, -0.290284677254461276,  0.881921264348356604,  0.634393284163646820, 
		-0.634393284163640381, -0.881921264348353828,  0.290284677254460111,  0.995184726672196263, 
		 0.098017140329558675, -0.956940335732207936, -0.471396736825992146,  0.773010453362737771, 
		 0.773010453362739325, -0.471396736825989982, -0.956940335732208713,  0.098017140329570387, 
		 0.995184726672195930,  0.290284677254462442, -0.881921264348359379, -0.634393284163653259, 
		 0.634393284163644933,  0.881921264348351053, -0.290284677254452117, -0.995184726672197040, 
		-0.098017140329552832,  0.956940335732205605,  0.471396736825999529, -0.773010453362741545, 
	},{
		-0.740951125354959106,  0.595699304492432691,  0.857728610000272451, -0.427555093430282973, 
		-0.941544065183019807,  0.242980179903264926,  0.989176509964781125, -0.049067674327415586, 
		-0.998795456205172183, -0.146730474455360582,  0.970031253194545529,  0.336889853392222216, 
		-0.903989293123444670, -0.514102744193226657,  0.803207531480644610,  0.671558954847014777, 
		-0.671558954847015444, -0.803207531480644055,  0.514102744193215333,  0.903989293123438120, 
		-0.336889853392223160, -0.970031253194545306,  0.146730474455375626,  0.998795456205172183, 
		 0.049067674327421699, -0.989176509964779127, -0.242980179903257071,  0.941544065183020140, 
		 0.427555093430291744, -0.857728610000276670, -0.595699304492433357,  0.740951125354953222, 
	}
};

int decode_layer2(LAYER2_HEADER *head, unsigned char *buffer, LAYER2_WORK *work, short *pcm)
{
	if(head->channel == 2){
		return decode_layer2_stereo(head, buffer, work, pcm);
	}else{
		return decode_layer2_monaural(head, buffer, work, pcm);
	}
}
	
static int decode_layer2_stereo(LAYER2_HEADER *head, unsigned char *buffer, LAYER2_WORK *work, short *pcm)
{
	int i,k,n,m;
	int step;
	int bits;
	int o1,o2;
	
	unsigned int code;
	
	int sb,ch,gr;

	int allocation[32][2];
	int scfsi[32][2];
	int scalefactor[32][2][3];
	double sample[12][3][32][2];

	double U[1024];
	double W[1024];

	double s[2];
	int ss[2];
	
	MEMORY_STREAM ms;

	const int *table_b2_nbal;
	const int **table_b2;

	ms_set_buffer(&ms, buffer, head->framesize);

	if(head->has_crc){
		ms_get_bits(&ms, 16);
	}

	if(head->sblimit > 20){
		table_b2_nbal = table_b2ab_nbal;
		table_b2 = table_b2ab;
	}else{
		table_b2_nbal = table_b2cd_nbal;
		table_b2 = table_b2cd;
	}

	/* read allocation */
	for(sb=0;sb<head->bound;sb++){
		for(ch=0;ch<2;ch++){
			n = ms_get_bits(&ms, table_b2_nbal[sb]);
			allocation[sb][ch] = table_b2[sb][n];
		}
	}

	for(sb=head->bound;sb<head->sblimit;sb++){
		n = ms_get_bits(&ms, table_b2_nbal[sb]);
		allocation[sb][0] = allocation[sb][1] = table_b2[sb][n];
	}
	
	/* read scalefactor selection information */
	for(sb=0;sb<head->sblimit;sb++){
		for(ch=0;ch<2;ch++){
			if(allocation[sb][ch]){
				scfsi[sb][ch] = ms_get_bits(&ms, 2);
			}
		}
	}

	/* read scalefactor */
	for(sb=0;sb<head->sblimit;sb++){
		for(ch=0;ch<2;ch++){
			if(allocation[sb][ch]){
				switch(scfsi[sb][ch]){
				case 0:
					scalefactor[sb][ch][0] = ms_get_bits(&ms, 6);
					scalefactor[sb][ch][1] = ms_get_bits(&ms, 6);
					scalefactor[sb][ch][2] = ms_get_bits(&ms, 6);
					break;
				case 1:
					scalefactor[sb][ch][0] = ms_get_bits(&ms, 6);
					scalefactor[sb][ch][1] = scalefactor[sb][ch][0];
					scalefactor[sb][ch][2] = ms_get_bits(&ms, 6);
					break;
				case 2:
					scalefactor[sb][ch][0] = ms_get_bits(&ms, 6);
					scalefactor[sb][ch][1] = scalefactor[sb][ch][0];
					scalefactor[sb][ch][2] = scalefactor[sb][ch][0];
					break;
				case 3:
					scalefactor[sb][ch][0] = ms_get_bits(&ms, 6);
					scalefactor[sb][ch][1] = ms_get_bits(&ms, 6);
					scalefactor[sb][ch][2] = scalefactor[sb][ch][1];
					break;
				}
			}
		}
	}

	/* read sample and requantize and normalize */
	for(gr=0;gr<12;gr++){
		for(sb=0;sb<head->bound;sb++){
			for(ch=0;ch<2;ch++){
				n = allocation[sb][ch];
				if(n == 0){
					for(i=0;i<3;i++){
						sample[gr][i][sb][ch] = 0;
					}
				}else if(n < 4){
					bits = table_b4[n].bits;
					step = table_b4[n].step;
					code = ms_get_bits(&ms, bits);
					for(i=0;i<3;i++){
						m = code % step;
						m -= (step-1);
						sample[gr][i][sb][ch] = m;
						sample[gr][i][sb][ch] /= (step-1);
						sample[gr][i][sb][ch] += table_b4[n].d;
						sample[gr][i][sb][ch] *= table_b4[n].c;
						sample[gr][i][sb][ch] *= table_b1[scalefactor[sb][ch][gr>>2]];
						code /= step;
					}
				}else{
					bits = table_b4[n].bits;
					step = table_b4[n].step;
					for(i=0;i<3;i++){
						m = ms_get_bits(&ms, bits);
						m -= step;
						sample[gr][i][sb][ch] = m;
						sample[gr][i][sb][ch] /= step;
						sample[gr][i][sb][ch] += table_b4[n].d;
						sample[gr][i][sb][ch] *= table_b4[n].c;
						sample[gr][i][sb][ch] *= table_b1[scalefactor[sb][ch][gr>>2]];
					}
				}
			}
		}
		for(sb=head->bound;sb<head->sblimit;sb++){
			n = allocation[sb][0];
			if(n == 0){
				for(i=0;i<3;i++){
					sample[gr][i][sb][0] = 0.0;
					sample[gr][i][sb][1] = 0.0;
				}
			}else if(n < 4){
				bits = table_b4[n].bits;
				step = table_b4[n].step;
				code = ms_get_bits(&ms, bits);
				for(i=0;i<3;i++){
					m = code % step;
					m -= (step-1);
					sample[gr][i][sb][0] = m;
					sample[gr][i][sb][0] /= (step-1);
					sample[gr][i][sb][0] += table_b4[n].d;
					sample[gr][i][sb][0] *= table_b4[n].c;
					sample[gr][i][sb][1] = sample[gr][i][sb][0];
					sample[gr][i][sb][0] *= table_b1[scalefactor[sb][0][gr>>2]];
					sample[gr][i][sb][1] *= table_b1[scalefactor[sb][1][gr>>2]];
					code /= step;
				}
			}else{
				bits = table_b4[n].bits;
				step = table_b4[n].step;
				for(i=0;i<3;i++){
					m = ms_get_bits(&ms, bits);
					m -= step;
					sample[gr][i][sb][0] = m;
					sample[gr][i][sb][0] /= step;
					sample[gr][i][sb][0] += table_b4[n].d;
					sample[gr][i][sb][0] *= table_b4[n].c;
					sample[gr][i][sb][1] = sample[gr][i][sb][0];
					sample[gr][i][sb][0] *= table_b1[scalefactor[sb][0][gr>>2]];
					sample[gr][i][sb][1] *= table_b1[scalefactor[sb][1][gr>>2]];
				}
			}
		}
	}

	/* subband synthesis */
	for(gr=0;gr<12;gr++){
		for(m=0;m<3;m++){
			/* shifting */
			work->offset -= 128;
			work->offset &= 2047;
			/* matrixing */
			for(i=0;i<64;i++){
				for(ch=0;ch<head->channel;ch++){
					work->area[work->offset+i*2+ch] = table_Nik[i][0] * sample[gr][m][0][ch];
					for(k=1;k<head->sblimit;k++){
						work->area[work->offset+i*2+ch] += table_Nik[i][k] * sample[gr][m][k][ch];
					}
				}
			}
			/* build vector U */
			for(i=0;i<8;i++){
				o1 = (work->offset + i*256) & 2047;
				o2 = (work->offset + i*256 + 192) & 2047;
				for(k=0;k<32;k++){
					U[(i*64+k)*2+0] = work->area[o1+k*2+0];
					U[(i*64+k)*2+1] = work->area[o1+k*2+1];
				}
				for(k=0;k<32;k++){
					U[(i*64+32+k)*2+0] = work->area[o2+k*2+0];
					U[(i*64+32+k)*2+1] = work->area[o2+k*2+1];
				}
			}
			/* window */
			for(i=0;i<512;i++){
				W[i*2+0] = U[i*2+0]*table_b3[i];
				W[i*2+1] = U[i*2+1]*table_b3[i];
			}

			/* calc sample */
			for(i=0;i<32;i++){
				s[0] = W[i*2+0];
				s[1] = W[i*2+1];
				for(k=1;k<16;k++){
					s[0] += W[(32*k+i)*2+0];
					s[1] += W[(32*k+i)*2+1];
				}
				ss[0] = (int)(s[0]*32768);
				ss[1] = (int)(s[1]*32768);
				if(ss[0] > 32767){
					pcm[0] = 32767;
				}else if(ss[0] < -32768){
					pcm[0] = -32768;
				}else{
					pcm[0] = (short)ss[0];
				}
				if(ss[1] > 32767){
					pcm[1] = 32767;
				}else if(ss[1] < -32768){
					pcm[1] = -32768;
				}else{
					pcm[1] = (short)ss[1];
				}
				pcm += 2;
			}
		}
	}

	return (ms.count+7)/8;
}

static int decode_layer2_monaural(LAYER2_HEADER *head, unsigned char *buffer, LAYER2_WORK *work, short *pcm)
{
	int i,k,n,m;
	int step;
	int bits;
	int o1,o2;
	
	unsigned int code;
	
	int sb,gr;

	int allocation[32];
	int scfsi[32];
	int scalefactor[32][3];
	double sample[12][3][32];

	double U[512];
	double W[512];

	double s;
	int    ss;
	
	MEMORY_STREAM ms;

	const int *table_b2_nbal;
	const int **table_b2;

	ms_set_buffer(&ms, buffer, head->framesize);

	if(head->has_crc){
		ms_get_bits(&ms, 16);
	}

	if(head->sblimit > 20){
		table_b2_nbal = table_b2ab_nbal;
		table_b2 = table_b2ab;
	}else{
		table_b2_nbal = table_b2cd_nbal;
		table_b2 = table_b2cd;
	}

	/* read allocation */
	for(sb=0;sb<head->bound;sb++){
		n = ms_get_bits(&ms, table_b2_nbal[sb]);
		allocation[sb] = table_b2[sb][n];
	}

	/* read scalefactor selection information */
	for(sb=0;sb<head->sblimit;sb++){
		if(allocation[sb]){
			scfsi[sb] = ms_get_bits(&ms, 2);
		}
	}

	/* read scalefactor */
	for(sb=0;sb<head->sblimit;sb++){
		if(allocation[sb]){
			switch(scfsi[sb]){
			case 0:
				scalefactor[sb][0] = ms_get_bits(&ms, 6);
				scalefactor[sb][1] = ms_get_bits(&ms, 6);
				scalefactor[sb][2] = ms_get_bits(&ms, 6);
				break;
			case 1:
				scalefactor[sb][0] = ms_get_bits(&ms, 6);
				scalefactor[sb][1] = scalefactor[sb][0];
				scalefactor[sb][2] = ms_get_bits(&ms, 6);
				break;
			case 2:
				scalefactor[sb][0] = ms_get_bits(&ms, 6);
				scalefactor[sb][1] = scalefactor[sb][0];
				scalefactor[sb][2] = scalefactor[sb][0];
				break;
			case 3:
				scalefactor[sb][0] = ms_get_bits(&ms, 6);
				scalefactor[sb][1] = ms_get_bits(&ms, 6);
				scalefactor[sb][2] = scalefactor[sb][1];
				break;
			}
		}
	}

	/* read sample and requantize and normalize */
	for(gr=0;gr<12;gr++){
		for(sb=0;sb<head->bound;sb++){
			n = allocation[sb];
			if(n == 0){
				for(i=0;i<3;i++){
					sample[gr][i][sb] = 0;
				}
			}else if(n < 4){
				bits = table_b4[n].bits;
				step = table_b4[n].step;
				code = ms_get_bits(&ms, bits);
				for(i=0;i<3;i++){
					m = code % step;
					m -= (step-1);
					sample[gr][i][sb] = m;
					sample[gr][i][sb] /= (step-1);
					sample[gr][i][sb] += table_b4[n].d;
					sample[gr][i][sb] *= table_b4[n].c;
					sample[gr][i][sb] *= table_b1[scalefactor[sb][gr>>2]];
					code /= step;
				}
			}else{
				bits = table_b4[n].bits;
				step = table_b4[n].step;
				for(i=0;i<3;i++){
					m = ms_get_bits(&ms, bits);
					m -= step;
					sample[gr][i][sb] = m;
					sample[gr][i][sb] /= step;
					sample[gr][i][sb] += table_b4[n].d;
					sample[gr][i][sb] *= table_b4[n].c;
					sample[gr][i][sb] *= table_b1[scalefactor[sb][gr>>2]];
				}
			}
		}
	}

	/* subband synthesis */
	for(gr=0;gr<12;gr++){
		for(m=0;m<3;m++){
			/* shifting */
			work->offset -= 64;
			work->offset &= 2047;
			/* matrixing */
			for(i=0;i<64;i++){
				work->area[work->offset+i] = table_Nik[i][0] * sample[gr][m][0];
				for(k=1;k<head->sblimit;k++){
					work->area[work->offset+i] += table_Nik[i][k] * sample[gr][m][k];
				}
			}
			/* build vector U */
			for(i=0;i<8;i++){
				o1 = (work->offset + i*128) & 2047;
				o2 = (work->offset + i*128 + 96) & 2047;
				for(k=0;k<32;k++){
					U[(i*64+k)] = work->area[o1+k];
				}
				for(k=0;k<32;k++){
					U[(i*64+32+k)] = work->area[o2+k];
				}
			}
			/* window */
			for(i=0;i<512;i++){
				W[i] = U[i]*table_b3[i];
			}

			/* calc sample */
			for(i=0;i<32;i++){
				s = W[i];
				for(k=1;k<16;k++){
					s += W[32*k+i];
				}
				ss = (int)(s*32768);
				if(ss > 32767){
					pcm[0] = 32767;
				}else if(ss < -32768){
					pcm[0] = -32768;
				}else{
					pcm[0] = (short)ss;
				}
				pcm += 1;
			}
		}
	}

	return (ms.count+7)/8;
}
